@@ -38,21 +38,21 @@ extern "C"
 namespace KExiv2Iface
 {
 
-KExiv2::KExiv2Priv::KExiv2Priv()
-    : data(new KExiv2Data::KExiv2DataPriv)
+KExiv2::Private::Private()
+    : data(new KExiv2Data::Private)
 {
     writeRawFiles         = false;
     updateFileTimeStamp   = false;
     useXMPSidecar4Reading = false;
     metadataWritingMode   = WRITETOIMAGEONLY;
-    Exiv2::LogMsg::setHandler(KExiv2::KExiv2Priv::printExiv2MessageHandler);
+    Exiv2::LogMsg::setHandler(KExiv2::Private::printExiv2MessageHandler);
 }
 
-KExiv2::KExiv2Priv::~KExiv2Priv()
+KExiv2::Private::~Private()
 {
 }
 
-void KExiv2::KExiv2Priv::copyPrivateData(const KExiv2Priv* const  other)
+void KExiv2::Private::copyPrivateData(const Private* const  other)
 {
     data                  = other->data;
     filePath              = other->filePath;
@@ -62,29 +62,29 @@ void KExiv2::KExiv2Priv::copyPrivateData(const KExiv2Priv* const  other)
     metadataWritingMode   = other->metadataWritingMode;
 }
 
-bool KExiv2::KExiv2Priv::saveToXMPSidecar(const QFileInfo& finfo) const
+bool KExiv2::Private::saveToXMPSidecar(const QFileInfo& finfo) const
 {
     QString filePath = KExiv2::sidecarFilePathForFile(finfo.filePath());
-    if (filePath.isEmpty())
-        return false;
 
-    bool ret = false;
+    if (filePath.isEmpty())
+    {
+        return false;
+    }
 
     try
     {
         Exiv2::Image::AutoPtr image;
         image = Exiv2::ImageFactory::create(Exiv2::ImageType::xmp, (const char*)(QFile::encodeName(filePath)));
-        ret   = saveOperations(image);
+        return saveOperations(finfo, image);
     }
     catch( Exiv2::Error& e )
     {
         printExiv2ExceptionError("Cannot save metadata to XMP sidecar using Exiv2 ", e);
+        return false;
     }
-
-    return ret;
 }
 
-bool KExiv2::KExiv2Priv::saveToFile(const QFileInfo& finfo) const
+bool KExiv2::Private::saveToFile(const QFileInfo& finfo) const
 {
     if (!finfo.isWritable())
     {
@@ -92,16 +92,34 @@ bool KExiv2::KExiv2Priv::saveToFile(const QFileInfo& finfo) const
         return false;
     }
 
-    QStringList rawTiffBasedSupported = QStringList()
-        // TIFF/EP Raw files based supported by Exiv2 0.19.1
-        << "dng" << "nef" << "pef" << "orf";
+    QStringList rawTiffBasedSupported, rawTiffBasedNotSupported;
 
-    QStringList rawTiffBasedNotSupported = QStringList()
-        // TIFF/EP Raw files based not supported by Exiv2 0.19.1
+    // Raw files supported by Exiv2 0.21
+    rawTiffBasedSupported << "dng" << "nef" << "pef" << "orf" << "srw";
+    if (Exiv2::testVersion(0,23,0))
+    {
+        rawTiffBasedSupported << "cr2";
+    }
+
+    // Raw files not supported by Exiv2 0.21
+    rawTiffBasedNotSupported
         << "3fr" << "arw" << "cr2" << "dcr" << "erf" << "k25"
-        << "kdc" << "mos" << "raw" << "sr2" << "srf";
+        << "kdc" << "mos" << "raw" << "sr2" << "srf" << "rw2";
+    if (!Exiv2::testVersion(0,23,0))
+    {
+        rawTiffBasedNotSupported << "cr2";
+    }
 
     QString ext = finfo.suffix().toLower();
+
+    if (!writeRawFiles && (rawTiffBasedSupported.contains(ext) || rawTiffBasedNotSupported.contains(ext)) )
+    {
+        kDebug() << finfo.fileName()
+                 << "is a TIFF based RAW file, writing to such a file is disabled by current settings.";
+        return false;
+    }
+
+/*
     if (rawTiffBasedNotSupported.contains(ext))
     {
         kDebug() << finfo.fileName()
@@ -116,29 +134,31 @@ bool KExiv2::KExiv2Priv::saveToFile(const QFileInfo& finfo) const
                  << "Metadata not saved.";
         return false;
     }
+
     kDebug() << "File Extension: " << ext << " is supported for writing mode";
 
     bool ret = false;
+*/
 
     try
     {
         Exiv2::Image::AutoPtr image;
         image = Exiv2::ImageFactory::open((const char*)(QFile::encodeName(finfo.filePath())));
-        ret   = saveOperations(image);
+        return saveOperations(finfo, image);
     }
     catch( Exiv2::Error& e )
     {
         printExiv2ExceptionError("Cannot save metadata to image using Exiv2 ", e);
+        return false;
     }
-
-    return ret;
 }
 
-bool KExiv2::KExiv2Priv::saveOperations(Exiv2::Image::AutoPtr image) const
+bool KExiv2::Private::saveOperations(const QFileInfo& finfo, Exiv2::Image::AutoPtr image) const
 {
     try
     {
         Exiv2::AccessMode mode;
+        bool wroteComment = false, wroteEXIF = false, wroteIPTC = false, wroteXMP = false;
 
         // We need to load target file metadata to merge with new one. It's mandatory with TIFF format:
         // like all tiff file structure is based on Exif.
@@ -147,14 +167,17 @@ bool KExiv2::KExiv2Priv::saveOperations(Exiv2::Image::AutoPtr image) const
         // Image Comments ---------------------------------
 
         mode = image->checkMode(Exiv2::mdComment);
+
         if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
             image->setComment(imageComments());
+            wroteComment = true;
         }
 
         // Exif metadata ----------------------------------
 
         mode = image->checkMode(Exiv2::mdExif);
+
         if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
             if (image->mimeType() == "image/tiff")
@@ -190,6 +213,7 @@ bool KExiv2::KExiv2Priv::saveOperations(Exiv2::Image::AutoPtr image) const
                 }
 
                 Exiv2::ExifData readedExif = exifMetadata();
+
                 for (Exiv2::ExifData::iterator it = readedExif.begin(); it != readedExif.end(); ++it)
                 {
                     if (!untouchedTags.contains(it->key().c_str()))
@@ -204,31 +228,46 @@ bool KExiv2::KExiv2Priv::saveOperations(Exiv2::Image::AutoPtr image) const
             {
                 image->setExifData(exifMetadata());
             }
+
+            wroteEXIF = true;
         }
 
         // Iptc metadata ----------------------------------
 
         mode = image->checkMode(Exiv2::mdIptc);
+
         if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
             image->setIptcData(iptcMetadata());
+            wroteIPTC = true;
         }
 
-#ifdef _XMP_SUPPORT_
-
         // Xmp metadata -----------------------------------
 
         mode = image->checkMode(Exiv2::mdXmp);
+
         if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
+#ifdef _XMP_SUPPORT_
             image->setXmpData(xmpMetadata());
+            wroteXMP = true;
+#endif
         }
 
-#endif // _XMP_SUPPORT_
+        if (!wroteComment && !wroteEXIF && !wroteIPTC && !wroteXMP)
+        {
+            kDebug() << "Writing metadata is not supported for file" << finfo.fileName();
+            return false;
+        }
+        else if (!wroteEXIF || !wroteIPTC || !wroteXMP)
+        {
+            kDebug() << "Support for writing metadata is limited for file" << finfo.fileName()
+                     << "EXIF" << wroteEXIF << "IPTC" << wroteIPTC << "XMP" << wroteXMP;
+        }
 
         if (!updateFileTimeStamp)
         {
-            // NOTE: Don't touch access and modification timestamp of file.
+            // Don't touch access and modification timestamp of file.
             struct stat st;
             ::stat(QFile::encodeName(filePath), &st);
 
@@ -255,7 +294,7 @@ bool KExiv2::KExiv2Priv::saveOperations(Exiv2::Image::AutoPtr image) const
     return false;
 }
 
-void KExiv2Data::KExiv2DataPriv::clear()
+void KExiv2Data::Private::clear()
 {
     imageComments.clear();
     exifMetadata.clear();
@@ -265,19 +304,19 @@ void KExiv2Data::KExiv2DataPriv::clear()
 #endif
 }
 
-void KExiv2::KExiv2Priv::printExiv2ExceptionError(const QString& msg, Exiv2::Error& e)
+void KExiv2::Private::printExiv2ExceptionError(const QString& msg, Exiv2::Error& e)
 {
     std::string s(e.what());
     kDebug() << msg.toAscii().constData() << " (Error #"
              << e.code() << ": " << s.c_str();
 }
 
-void KExiv2::KExiv2Priv::printExiv2MessageHandler(int lvl, const char* msg)
+void KExiv2::Private::printExiv2MessageHandler(int lvl, const char* msg)
 {
     kDebug() << "Exiv2 (" << lvl << ") : " << msg;
 }
 
-QString KExiv2::KExiv2Priv::convertCommentValue(const Exiv2::Exifdatum& exifDatum)
+QString KExiv2::Private::convertCommentValue(const Exiv2::Exifdatum& exifDatum) const
 {
     try
     {
@@ -327,7 +366,7 @@ QString KExiv2::KExiv2Priv::convertCommentValue(const Exiv2::Exifdatum& exifDatu
     return QString();
 }
 
-QString KExiv2::KExiv2Priv::detectEncodingAndDecode(const std::string& value) const
+QString KExiv2::Private::detectEncodingAndDecode(const std::string& value) const
 {
     // For charset autodetection, we could use sophisticated code
     // (Mozilla chardet, KHTML's autodetection, QTextCodec::codecForContent),
@@ -336,7 +375,9 @@ QString KExiv2::KExiv2Priv::detectEncodingAndDecode(const std::string& value) co
     // TODO: Gilles ==> Marcel : Look like KEncodingDetector class can provide a full implementation for encoding detection.
 
     if (value.empty())
+    {
         return QString();
+    }
 
     if (KStringHandler::isUtf8(value.c_str()))
     {
@@ -353,7 +394,7 @@ QString KExiv2::KExiv2Priv::detectEncodingAndDecode(const std::string& value) co
     //return QString::fromLatin1(value.c_str());
 }
 
-int KExiv2::KExiv2Priv::getXMPTagsListFromPrefix(const QString& pf, KExiv2::TagsMap& tagsMap) const
+int KExiv2::Private::getXMPTagsListFromPrefix(const QString& pf, KExiv2::TagsMap& tagsMap) const
 {
     QList<const Exiv2::XmpPropertyInfo*> tags;
     tags << Exiv2::XmpProperties::propertyList(pf.toAscii().data());
@@ -371,15 +412,17 @@ int KExiv2::KExiv2Priv::getXMPTagsListFromPrefix(const QString& pf, KExiv2::Tags
             i++;
         }
     }
+
     return i;
 }
 
 #ifdef _XMP_SUPPORT_
-void KExiv2::KExiv2Priv::mergeXmpData(const Exiv2::XmpData& src, Exiv2::XmpData& dest)
+void KExiv2::Private::mergeXmpData(const Exiv2::XmpData& src, Exiv2::XmpData& dest)
 {
     for (Exiv2::XmpData::const_iterator it = src.begin(); it != src.end(); ++it)
     {
         Exiv2::XmpData::iterator destIt = dest.findKey(Exiv2::XmpKey(it->key()));
+
         if (destIt == dest.end())
         {
             dest.add(*it);
